"
 COPYRIGHT (c) 1989 by Claus Gittinger
	      All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:libtool' }"

SimpleView subclass:#InspectorView
	instanceVariableNames:'listView labelView workspace inspectedObject selectedLine nShown
		hasMore monitorProcess hideReceiver showHex inspectHistory
		allowFollow isStandaloneInspector selectionIndex object
		inspectedObjectHolder displayStringMessage suppressPseudoSlots
		dereferenceValueHolders'
	classVariableNames:'DefaultIcon IdDictionary NextSequentialID LastExtent'
	poolDictionaries:''
	category:'Interface-Inspector'
!

!InspectorView class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 1989 by Claus Gittinger
	      All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    This class implements a graphical inspector.
    Inspecting can be done on an object -
    (where its instvarnames/values are inspected)
    or a list of objects (where a nameArray/valuesArray is inspected).
    The later is used by the debugger to inspect method variables/args.

    The system calls the inspector through the global variable 'Inspector'
    which is bound to this class (but could be redefined - it actually is
    set to MiniInspector in a smalltalk without graphical user interface,
    or to NewInspector if that is wanted).

    Also notice, that there are two methods to inspect an object:
    sending #basicInspect to any object, will open this kind of inspector on 
    it (showing instance variables which are physically present).

    Sending it #inspect, will - depending on the object - sometimes invoke a
    specialized inspector. 
    (see OrderedCollectionInspectorView, ImageInspectorView, 
     ColorInspectorView etc. as examples).

    You can also open a monitoring inspector, which displays some instance
    variable in regular intervals. See #openOn:monitor:.

    examples:
            #(1 2 3 4) asOrderedCollection inspect
            #(1 2 3 4) asOrderedCollection basicInspect
            (Array new:10000) inspect
            (Image fromFile:'bitmaps/claus.gif') inspect
            (Image fromFile:'bitmaps/claus.gif') basicInspect
            (Image fromFile:'bitmaps/SBrowser.xbm') inspect
            (Image fromFile:'bitmaps/SBrowser.xbm') basicInspect

            InspectorView openOn:Display
            InspectorView openOn:Display monitor:'shiftDown'

    The InspectorView can also be used as a subComponent within another view.
    In this case, the isStandAlone flag should be cleared, to prevent the
    inspector from changing the topViews window label.

    Notice:
        the instvars 'inspectedObject' and 'selectedLine' have been 
        renamed to 'object' and 'selectionIndex' for squeak compatibility;
        however, the old vars are kept (in sync) for a while, to allow for
        smooth migration.

  Controlling the contents from the inspected object.

    By redefining inspectorExtraAttributes or inspectorExtraNamedFields, the inspected
    object can add items to the list of fields as ashown in the left list-view of the inspector.
    These methods are meant to return a sequencable Collection of Associations, which represent
    of pseudo slot-name, slot-value pairs.
    In the list, extra attributes are shown with a dash (-), extra named fields are marked with a tick (`).
    These are added (read only) to the list.

    [author:]
        Claus Gittinger
"
! !

!InspectorView class methodsFor:'instance creation'!

for:anObject
    "create and launch a new inspector for anObject.
     This protocol is a historic leftover - this method will vanish."

    ^ self openOn:anObject
!

inspect:anObject
    "create and launch a new inspector for anObject.
     This protocol is a historic leftover - this method will vanish."

    ^ self openOn:anObject
!

openOn:anObject
    "create and launch a new inspector for anObject"

    ^ self openOn:anObject monitor:nil

    "
     InspectorView openOn:(5 @ 7)
     InspectorView openOn:(Array new:400)
     DictionaryInspectorView openOn:(IdentityDictionary new)
    "

    "Modified: 1.3.1996 / 19:31:03 / cg"
!

openOn:anObject monitor:anInstVarName
    "create and launch a new inspector for anObject.
     If anInstVarName is nonNil, let the inspector monitor it
     (use an integer-printString as name, for indexed instVars)."

    |topView inspectorView|

    topView := StandardSystemView new.
    topView icon:self defaultIcon.
    "/ topView minExtent:(100 @ 100).
    topView label:'Inspector'.
    topView iconLabel:'Inspector'.

    topView extent:(self defaultTopViewExtent).
    topView objectAttributeAt:#rememberExtent put:true.

    inspectorView := self origin:(0.0 @ 0.0)
                          corner:(1.0 @ 1.0)
                             in:topView.

    "kludge: must realize first, to be able to set menu again"
    topView openAndWait.
    topView windowGroup 
        focusSequence:(Array 
                            with:inspectorView listView
                            with:inspectorView workspace).
    inspectorView allowFollow:true.
    inspectorView isStandaloneInspector:true.
    inspectorView inspect:anObject.

    anInstVarName notNil ifTrue:[
        inspectorView monitor:anInstVarName
    ].

    ^ inspectorView

    "
     |m|

     m := 1 asValue.
     InspectorView openOn:m monitor:'value'.

     2 to:10 do:[:i |
         Delay waitForSeconds:1.
         m value:i
     ]
    "

    "
     |o|

     o := Array with:1 with:2 with:3.
     InspectorView openOn:o monitor:'2'.
     Delay waitForSeconds:1.
     o at:2 put:20
    "

    "Created: / 01-03-1996 / 19:30:50 / cg"
    "Modified: / 23-10-2007 / 19:08:21 / cg"
!

openOn:anObject withEvalPane:withEvalPane
    ^ self openOn:anObject
!

openOn:anObject withEvalPane:withEvalPane withLabel:aLabel
    ^ self openOn:anObject
! !

!InspectorView class methodsFor:'defaults'!

defaultExtent
    ^ (Screen current extent // 3)

    "Created: / 7.9.1998 / 13:47:45 / cg"
    "Modified: / 7.9.1998 / 14:15:38 / cg"
!

defaultIcon
    "return the browsers default window icon"

    <resource: #programImage>
    <resource: #style (#INSPECTOR_ICON #INSPECTOR_ICON_FILE)>

    |nm i resources|

    (i := DefaultIcon) isNil ifTrue:[
        resources := self classResources.
        i := resources at:#INSPECTOR_ICON default:nil.
        i isNil ifTrue:[
            nm := resources at:#INSPECTOR_ICON_FILE default:'Inspector.xbm'.
            i := Smalltalk imageFromFileNamed:nm forClass:self.
            i isNil ifTrue:[
                i := StandardSystemView defaultIcon
            ]
        ].
        i notNil ifTrue:[
            DefaultIcon := i := i onDevice:Display
        ]
    ].
    ^ i

    "
       DefaultIcon := nil
    "

    "Modified: / 17-09-2007 / 11:36:17 / cg"
!

defaultTopViewExtent
    ^ LastExtent ? self defaultExtent

    "Created: / 23-10-2007 / 19:04:13 / cg"
!

rememberLastExtent:anExtent
    LastExtent := anExtent

    "Created: / 23-10-2007 / 19:10:02 / cg"
! !

!InspectorView class methodsFor:'queries-plugin'!

aspectSelectors
    ^ #( inspectedObjectHolder )

    "Modified: / 10.2.2000 / 12:25:28 / cg"
! !

!InspectorView methodsFor:'accessing'!

allowFollow:aBoolean
    "enable/disable the follow menu item;
     This is disabled for inspectors which are embedded in the debugger"

    allowFollow := aBoolean
!

dereferenceValueHolders:aBoolean
    dereferenceValueHolders := aBoolean
!

fieldListLabel:aString
    labelView label:aString

    "Created: 28.6.1996 / 15:30:26 / cg"
!

hideReceiver:aBoolean
    "hide/show the self-entry for the inspected object;
     This is hidden for context inspectors in the debugger"

    hideReceiver := aBoolean

    "Created: 28.6.1996 / 15:08:32 / cg"
!

inspect:anObject
    "define the object to be inspected"

    |aList sameObject|

    sameObject := anObject == inspectedObject and:[inspectedObject notNil].
    inspectedObject := object := anObject.

    sameObject ifFalse:[
        hasMore := false.
        aList := self fieldList.                               
        hasMore ifTrue:[
            aList add:' ... '
        ].
    ].

    sameObject ifTrue:[
"/        listView setContents:aList.
    ] ifFalse:[
        listView contents:aList.

        workspace contents:nil.
        self setDoItAction.

    ].
    selectionIndex := selectedLine := nil.

    isStandaloneInspector == true ifTrue:[
        self topView 
            label:(self labelFor:anObject);
            iconLabel:(self labelNameFor:anObject).
    ]

    "Modified: / 05-11-2007 / 20:09:14 / cg"
!

isStandaloneInspector:aBoolean
    "obsolete now"

    isStandaloneInspector := aBoolean

    "Modified: / 12.2.1999 / 16:01:44 / cg"
!

label:aString
    "set the fieldListLabel - obsolete; collides with inherited label-functionality"

    <resource:#obsolete>
    self obsoleteMethodWarning:'use fieldListLabel:'.
    self fieldListLabel:aString.
    super label:aString.

    "Created: 28.6.1996 / 15:30:26 / cg"
!

listView
    ^ listView
!

reinspect
    "update display for a changed inspectedObject"

    |aList|

    hasMore := false.
    aList := self fieldList.
    hasMore ifTrue:[
        aList add:' ... '
    ].

    listView contents:aList.

"/    workspace contents:nil.
    self setDoItAction.

    selectionIndex := selectedLine := nil
!

suppressPseudoSlots:aBoolean
    suppressPseudoSlots := aBoolean
!

workspace
    ^ workspace
! !

!InspectorView methodsFor:'accessing-channels'!

inspectedObjectHolder
    inspectedObjectHolder isNil ifTrue:[
        inspectedObjectHolder := ValueHolder new.
        inspectedObjectHolder addDependent:self.
    ].
    ^ inspectedObjectHolder

    "Created: / 10.2.2000 / 13:33:16 / cg"
    "Modified: / 10.2.2000 / 13:34:23 / cg"
!

inspectedObjectHolder:aValueHolder
    inspectedObjectHolder notNil ifTrue:[
        inspectedObjectHolder removeDependent:self.
    ].
    inspectedObjectHolder := aValueHolder.
    inspectedObjectHolder notNil ifTrue:[
        inspectedObjectHolder addDependent:self.
    ].

    "Created: / 10.2.2000 / 13:34:53 / cg"
! !

!InspectorView methodsFor:'change & update'!

update:something with:aParameter from:changedObject
    "Invoked when one of my dependees sends a change notification."

    |oldSelection|

    changedObject == inspectedObject ifTrue:[
        oldSelection := listView selection.
        self inspect:inspectedObject.
        oldSelection notNil ifTrue:[
            self showSelection:oldSelection
        ]
    ].

    changedObject == inspectedObjectHolder ifTrue:[
        self inspect:(inspectedObjectHolder value)
    ].

    super update:something with:aParameter from:changedObject

    "Created: / 10.2.2000 / 13:46:38 / cg"
    "Modified: / 10.2.2000 / 13:48:18 / cg"
! !

!InspectorView methodsFor:'drag & drop'!

getDisplayObjects

    ^List with:self selection printString

    "Created: / 16-08-2005 / 21:54:52 / janfrog"
    "Modified: / 18-09-2006 / 21:11:16 / cg"
!

getDropObjects

    ^List with:(DropObject new:self selection).

    "Created: / 16-08-2005 / 21:49:23 / janfrog"
    "Modified: / 18-09-2006 / 21:11:31 / cg"
! !

!InspectorView methodsFor:'initialization & release'!

destroy
    (self topView objectAttributeAt:#rememberExtent) == true ifTrue:[
        self class rememberLastExtent:(self topView extent).
    ].

    inspectedObject := object := nil.
    monitorProcess notNil ifTrue:[
        monitorProcess terminate
    ].
    super destroy

    "Modified: / 23-10-2007 / 19:11:04 / cg"
!

initialize
    |v panel helpView labelView2|

    super initialize.

    displayStringMessage := #displayString.
    hideReceiver := false.
    showHex := false.
    allowFollow := false.
    isStandaloneInspector := false.

    panel := VariableHorizontalPanel 
                origin:(0.0 @ 0.0)
                corner:(1.0 @ 1.0)
                in:self.

    helpView := View origin:(0.0 @ 0.0) corner:(0.3 @ 1.0) in:panel.
    helpView level:0; borderWidth:0.

    labelView := Label origin:0.0@0.0 corner:1.0@0.0 in:helpView.
    labelView label:(self defaultLabel).
    labelView bottomInset:(labelView preferredExtent y negated).

    v := HVScrollableView 
                for:SelectionInListView 
                miniScrollerH:true
                miniScrollerV:false
                in:helpView.
"/    v autoHideScrollBars:true.
    v origin:(0.0 @ 0.0) corner:(1.0 @ 1.0).
    v topInset:(labelView preferredExtent y).

    listView := v scrolledView.
    listView action:[:lineNr | self showSelection:lineNr].
    listView doubleClickAction:[:lineNr | self doInspect].
    listView ignoreReselect:false.
    listView menuHolder:self; menuPerformer:self; menuMessage:#fieldMenu.
    self initializeDragAndDrop.

    helpView := View origin:(0.3 @ 0.0) corner:(1.0 @ 1.0) in:panel.
    helpView level:0; borderWidth:0.

    labelView2 := Label origin:0.0@0.0 corner:1.0@0.0 in:helpView.
    labelView2 label:''.
    labelView2 bottomInset:(labelView preferredExtent y negated).

    v := HVScrollableView 
                for:CodeView 
                miniScrollerH:true
                miniScrollerV:false
                in:helpView.
"/    v autoHideScrollBars:true.
    v origin:(0.0 @ 0.0) corner:(1.0 @ 1.0).
    v topInset:(labelView2 preferredExtent y).
    workspace := v scrolledView.

    self setAcceptAction.

    nShown := 100.
    hasMore := false.

    "Modified: / 16-08-2005 / 21:54:04 / janfrog"
    "Modified: / 18-09-2006 / 21:12:42 / cg"
!

initializeDragAndDrop

    | source |
    source := DropSource 
                    receiver:self
                    argument:nil
                    dropObjectSelector:#getDropObjects
                    displayObjectSelector:#getDisplayObjects
                    dropFeedBackSelector:nil.

    listView dropSource:source.

    "Created: / 16-08-2005 / 21:51:43 / janfrog"
    "Modified: / 18-09-2006 / 21:13:05 / cg"
!

realize
    "delayed setup of lists till first map-time - 
     this makes startup of inspectors a bit faster"

    |o|

    super realize.
    "/ cg: I dont remember what this was needed for (is it still?)
    false "inspectedObject notNil" ifTrue:[
        "
         kludge to trick inspect:, which ignores setting the
         same object again ...
        "
        o := inspectedObject.
        inspectedObject := object := nil.
        self inspect:o
    ]

    "Created: / 30-05-1996 / 09:38:37 / cg"
    "Modified: / 05-11-2007 / 20:11:44 / cg"
!

release
    "release inpected object. This is normally not needed,
     since the garbage collector will find this memory alone.
     However, if some applications keeps invisible inspectors around
     (for example, the debugger does this), the inspected object
     would be kept from being freed or freed very late."

"
    inspectedObject notNil ifTrue:[
        inspectedObject removeDependent:self
    ].
"
    inspectedObject := object := nil.
    self setDoItAction.      "/ to release ref to inspectedObject in doItBlock
    workspace contents:nil.
    listView contents:nil.

    super release.

    "Modified: 11.6.1997 / 13:20:39 / cg"
! !

!InspectorView methodsFor:'menu'!

fieldMenu
    "return the menu for the field-list"

    <resource: #programMenu>

    |items m sel|

    items := #(
                       ('Copy Name or Key'             #doCopyKey              )
                       ('-')
                       ('Inspect'                      #doInspect              )
                       ('BasicInspect'                 #doBasicInspect         )
                       ('Inspect Hierarchical'         #doNewInspect           )
                       ('-')
                       ('Ref Chains'                   #showReferences         )
                       ('-')
                       ('Browse'                       #browse                 )
"/                       ('Browse class hierarchy'       #browseClassHierarchy   )
"/                       ('Browse full class protocol'   #browseFullClassProtocol)
              ).

    sel := self selection.

    (sel isBlock or:[sel isContext]) ifTrue:[
        items := items , #(
                       ('Browse Blocks Home'           #browseHome)
              ).
    ].
    (inspectedObject isMethod or:[sel isMethod]) ifTrue:[
        items := items , #(
                       ('Browse Methods Class'         #browseMethodsClass)
                 ).
    ].
    items := items , (self optionalStreamSelectionItems).
    items := items , (self optionalFilenameSelectionItems).

    items := items , #(
                       ('-')
                       ('Trap Message...'              #doTrap                 )
                       ('Trap all Messages'            #doTrapAll              )
                       ('Trace all Messages'           #doTraceAll             )
                       ('Untrace/Untrap'               #doUntrace              )
"/                       ('-')
"/                       ('Trap change to instVar'       #doTrapInstVarChange    )
"/                       ('Trap change to any instVar'   #doTrapAnyInstVarChange )
              ).

    allowFollow ifTrue:[
        items := #(
                            ('Follow'                       #doFollow              )
                            ('Back'                         #doBack              )
                            ('-')
                  )
                 ,
                 items.
    ].

    hasMore ifTrue:[
        items := items , #(
                        ('-')
                        ('Show More'                    #showMore               )
                          )
    ].

    monitorProcess isNil ifTrue:[
        items := items , #(
                        ('-')
                        ('Start Monitor'                #doStartMonitor         )
                          ).
    ] ifFalse:[
        items := items , #(
                        ('-')
                        ('Stop Monitor'                #doStopMonitor           )
                          ).
    ].

    items := items , (self numberBaseItems).

    m := PopUpMenu
          itemList:items
          resources:resources.

    selectionIndex isNil ifTrue:[
        m disableAll:#(doFollow doInspect doBasicInspect doNewInspect
                       browse browseClassHierarchy browseFullClassProtocol
                       doStartMonitor doCopyKey doCopyKey)
    ].
    (selectionIndex == 1) ifTrue:[
        m disableAll:#(doFollow doCopyKey)
    ].
    inspectHistory size == 0 ifTrue:[
        m disable:#doBack
    ].
    sel class hasImmediateInstances ifTrue:[
        m disableAll:#(showReferences doNewInspect)
    ].
    sel inspectorClass == self class ifFalse:[
        m disable:#doFollow
    ].
    sel isMethod ifFalse:[
        m disable:#browseMethodsClass
    ].

    ^ m

    "Modified: / 09-02-2007 / 16:11:08 / cg"
!

numberBaseItems
    showHex ifTrue:[
        ^ #(
                        ('-')
                        ('Decimal Integers'  #toggleHex               )
                          )
    ] ifFalse:[
        ^ #(
                        ('-')
                        ('Hex Integers'      #toggleHex               )
                          )
    ].

    "Created: / 09-02-2007 / 16:07:31 / cg"
!

optionalFilenameSelectionItems
    |sel|

    sel := self selection.
    (inspectedObject isFilename or:[sel isFilename]) ifTrue:[
        ^ #(
                       ('Open FileBrowser'             #openFileBrowser)
              ).
    ].
    ^ #()

    "Created: / 09-02-2007 / 16:10:30 / cg"
!

optionalStreamSelectionItems
    |sel|

    sel := self selection.

    (sel isStream and:[self isExternalStream not]) ifTrue:[
        ^ #(
                       ('-')
                       ('Show Stream Contents'         #showStreamContents)
              ).
    ].
    ^ #()

    "Created: / 09-02-2007 / 16:09:15 / cg"
! !

!InspectorView methodsFor:'menu actions'!

browse
    |cls|

    cls := self selection class.
    cls browserClass openInClass:cls selector:nil

    "Created: / 14.12.1995 / 19:15:50 / cg"
    "Modified: / 14.10.1998 / 15:32:10 / cg"
!

browseClass
    |cls|

    cls := self selection class.
    cls browserClass browseClass:cls

    "Modified: 3.5.1996 / 12:39:32 / cg"
!

browseClassHierarchy
    |cls|

    cls := self selection class.
    cls browserClass browseClassHierarchy:cls

    "Modified: 3.5.1996 / 12:40:04 / cg"
!

browseFullClassProtocol
    |cls|

    cls := self selection class.
    cls browserClass browseFullClassProtocol:cls

    "Modified: 3.5.1996 / 12:40:17 / cg"
!

browseHome
    |sel mthd|

    sel := self selection.
    sel isBlock ifTrue:[
        mthd := sel homeMethod
    ] ifFalse:[
        sel isContext ifTrue:[
            mthd := sel method.
        ]
    ].
    mthd isNil ifTrue:[
        ^ self warn:'Sorry - cannot figure out home method.'
    ].
    mthd class browserClass 
        openInClass:mthd mclass selector:mthd selector
!

browseMethodsClass
    |mthd|

    mthd := self selection.
    mthd mclass browserClass 
        openInClass:mthd mclass selector:mthd selector
!

doBack
    "user selected back-menu entry"

    |objectToInspect|

    inspectHistory size > 0 ifTrue:[
	objectToInspect := inspectHistory removeLast.
	inspectHistory size == 0 ifTrue:[
	    inspectHistory := nil
	].
	self inspect:objectToInspect.
    ]

    "Created: / 22.9.1998 / 18:22:01 / cg"
    "Modified: / 22.9.1998 / 18:22:28 / cg"
!

doBasicInspect
    "user selected inspect-menu entry"

    self doInspect:true 
!

doFollow
    "user selected follow-menu entry"

    |objectToInspect|

    selectionIndex notNil ifTrue:[
        objectToInspect := self selection.
        inspectHistory isNil ifTrue:[
            inspectHistory := OrderedCollection new
        ].
        inspectHistory addLast:inspectedObject.
        self inspect:objectToInspect.
    ]

    "Created: / 22.9.1998 / 18:21:08 / cg"
    "Modified: / 22.9.1998 / 18:22:23 / cg"
!

doInspect
    "user selected inspect-menu entry"

    self doInspect:false
!

doNewInspect
    self doInspect:#new

    "Created: / 31.10.1997 / 12:45:38 / cg"
    "Modified: / 31.10.1997 / 12:47:11 / cg"
!

doStartMonitor
    "start a monitoring process"

    monitorProcess isNil ifTrue:[
        monitorProcess :=
            [
                |sel|

                [true] whileTrue:[
                    (sel := selectionIndex) notNil ifTrue:[
                        self showSelection:sel 
                    ].
                    (Delay forSeconds:0.5) wait
                ]
            ] forkAt:Processor userBackgroundPriority
    ]

    "Modified: 12.4.1996 / 14:20:06 / cg"
!

doStopMonitor
    "stop the monitor"

    monitorProcess terminate.
    monitorProcess := nil
!

doTraceAll
    "place a trace on all messages sent to the inspected object"

    self topView withWaitCursorDo:[MessageTracer traceAll:inspectedObject on:Transcript]
!

doTrap
    "place a trap on a message sent to the inspected object"

    |string selector|

    string := Dialog request:'Selector to trap on:' onCancel:nil.

    string notNil ifTrue:[
	selector := string asSymbolIfInterned.

	selector isNil ifTrue:[
	    self warn:'no such selector'
	] ifFalse:[
	    self topView withWaitCursorDo:[MessageTracer trap:inspectedObject 
							 selector:selector]
	]
    ]

    "Modified: 12.4.1996 / 14:07:01 / cg"


!

doTrapAll
    "place a trap on all messages sent to the inspected object"

    self topView withWaitCursorDo:[MessageTracer trapAll:inspectedObject]
!

doTrapAnyInstVarChange
    "place a trap which is triggered if any instVar of the inspected object is changed"

    self topView withWaitCursorDo:[
        MessageTracer trapModificationsIn:inspectedObject
    ]
!

doTrapInstVarChange
    "place a trap which is triggered if the selected instVar of the inspected object is changed"

    self topView withWaitCursorDo:[
        |idx|

        "/ a named instVar ?
        idx := self instVarIndexForLine:selectionIndex.
        idx isNil ifTrue:[
            self warn:'select an instance variable first.'.
            ^ self.
        ].
        MessageTracer 
                trapModificationsOf:(inspectedObject class allInstVarNames at:idx)
                in:inspectedObject
    ]
!

doUntrace
    "remove traps/traces"

    MessageTracer untrace:inspectedObject
!

inspectOwners
    "open an inspector on owners of the inspectedObject.
     (this is a secret function)"

    self withCursor:(Cursor questionMark) do:[
	|owners dict|

	owners := (ObjectMemory whoReferences:inspectedObject) asOrderedCollection.
	owners size > 500 ifTrue:[
	    (self confirm:'there are ' , owners size printString , ' owners.\\Do you really want to see them all ?' withCRs)
	    ifFalse:[^ self]
	].
	dict := IdentityDictionary new.
	owners do:[:owner |
	    |set names oClass|

	    owner ~~ self ifTrue:[
		set := Set new.
		names := owner class allInstVarNames.
		oClass := owner class.
		1 to:oClass instSize do:[:i |
		    (owner instVarAt:i) == inspectedObject ifTrue:[
			set add:(names at:i).
		    ].
		].
		oClass isVariable ifTrue:[
		    oClass isPointers ifTrue:[
			1 to:owner basicSize do:[:i |
			    (owner basicAt:i) == inspectedObject ifTrue:[
				 set add:i
			    ]
			]
		    ]
		].
		dict at:owner put:set
	    ].
	].
	dict inspect
    ]
!

openFileBrowser
    |fn|

    fn := self selection.
    (UserPreferences current fileBrowserClass) openOnFileNamed:fn.
!

showStreamContents
    |sel|

    sel := self selection.
    (sel isStream and:[sel isExternalStream not]) ifTrue:[
        workspace replace:(sel contents printString)
    ].

    "Created: / 6.2.2000 / 13:46:37 / cg"
    "Modified: / 6.2.2000 / 13:47:37 / cg"
! !

!InspectorView methodsFor:'private'!

baseInspectedObjectClass
    ^ Object


!

basicDisplayStringForValue:someValue 
    "return the values displayString"

    |s|

    Error handle:[:ex |
        s := someValue classNameWithArticle.
        displayStringMessage == #displayString ifTrue:[
            s := s , ' "error in displayString: ' , ex description , '"'
        ] ifFalse:[
            s := s , ' "error in displayString (' , displayStringMessage , '): ' , ex description , '"'
        ]
    ] do:[
        showHex ifTrue:[
            someValue isInteger ifTrue:[
                ^ '16r' , someValue hexPrintString
            ].
            (someValue isMemberOf:ByteArray) ifTrue:[
                ^ String streamContents:[:s | inspectedObject asByteArray printOn:s base:16 showRadix:true]
"/                    s := '' writeStream.
"/                    s nextPutAll:'#['.
"/                    someValue keysAndValuesDo:[:i :byte |
"/                        i ~~ 1 ifTrue:[
"/                            s space
"/                        ].
"/                        s nextPutAll:'16r'.
"/                        s nextPutAll:(byte hexPrintString leftPaddedTo:2 with:$0).
"/                    ].
"/                    s nextPutAll:']'.
"/                    s := s contents
            ]
        ].

        "/ displayStringMessage := #classNameWithArticle
        "/ displayStringMessage := #displayString
        "/ displayStringMessage := #printString
        "/ mhmh - avoid sending #perform: (bad for proxy objects which pass it to somewhere..)
        displayStringMessage == #displayString ifTrue:[
            s := someValue displayString.
        ] ifFalse:[
            displayStringMessage == #printString ifTrue:[
                s := someValue printString.
            ] ifFalse:[
                displayStringMessage == #storeString ifTrue:[
                    s := someValue storeString.
                ] ifFalse:[
                    s := someValue perform:displayStringMessage.
                ].
            ].
        ].
    ].
    ^ s.

    "Modified: / 05-11-2007 / 20:07:21 / cg"
!

defaultLabel
    ^ 'InstVars'

    "Modified: 28.6.1996 / 16:04:53 / cg"
!

derivedFieldNames
    |d|

    d := self derivedFields.
    (d isKindOf:Dictionary) ifTrue:[
        ^ d keys asSortedCollection.
    ].
    ^ d collect:[:eachEntry |
            |nm|

            nm := (eachEntry isAssociation) 
                    ifTrue:[ eachEntry key ] 
                    ifFalse:[ eachEntry first ].
            '-',nm
        ]

    "Created: / 03-08-2006 / 15:02:54 / cg"
    "Modified: / 18-09-2006 / 21:18:57 / cg"
!

derivedFields
    "the check below is not sufficient - if someone catches messages, for example.
     Therefore, we do a manual lookup here:"
    (object class whichClassImplements:#inspectorExtraAttributes) isNil ifTrue:[
        ^ #()
    ].
    ^ [object inspectorExtraAttributes]
                on: MessageNotUnderstood
                do: [:ex | ex return: #() ]

    "Created: / 17-07-2006 / 11:02:32 / cg"
    "Modified: / 29-08-2006 / 13:03:31 / cg"
!

displayStringForValue:someValue 
    "return the values displayString"

    |sel radix extraAttributes|

    selectionIndex notNil ifTrue:[
        sel := listView at:selectionIndex.

        extraAttributes := inspectedObject inspectorExtraAttributes.
        (extraAttributes notNil and:[extraAttributes includesKey:sel]) ifTrue:[
            ^ (extraAttributes at:sel) value printString
        ].

        (sel startsWith:'-all inst vars') ifTrue:[
            ^ self stringWithAllInstVarValues
        ].
        (sel startsWith:'-all indexed vars') ifTrue:[
            ^ self stringWithAllIndexedVarValues
        ].
    ].
    ^ self basicDisplayStringForValue:someValue

    "Modified: / 05-11-2007 / 20:06:44 / cg"
!

extraNamedFieldNames
    ^ self extraNamedFields 
        collect:[:eachEntry |
            |nm|

            nm := (eachEntry isAssociation) 
                    ifTrue:[ eachEntry key ] 
                    ifFalse:[ eachEntry first ].
            '`',nm
        ]

    "Modified: / 03-08-2006 / 15:17:19 / cg"
!

extraNamedFields
    "by redefining inspectorExtraNamedFields to return an array of
     pseudo-fieldName->value associations, the inspectors left list can be extended"

    "the check below is not sufficient - if someone catches messages, for example.
     Therefore, we do a manual lookup here:"
    (object class whichClassImplements:#inspectorExtraNamedFields) isNil ifTrue:[
        ^ #()
    ].
    ^ [object inspectorExtraNamedFields]
                on: MessageNotUnderstood
                do: [:ex | ex return: #() ]

    "Created: / 03-08-2006 / 13:34:18 / cg"
    "Modified: / 29-08-2006 / 13:03:57 / cg"
!

extraNamedVarIndexForLine:lineNr
    "helper - return the index for a named instVar;
     nil, if self or a keyed instvar is selected."

    |idx nNamedInstvarsShown nExtraNamedInstvarsShown cls baseCls firstRealIndex line|

    lineNr isNil ifTrue:[^ nil].
    firstRealIndex := 1.

    idx := lineNr.
    self hasSelfEntry ifTrue:[
        (lineNr == 1 or:[lineNr isNil]) ifTrue:[
            ^ nil "/ self selected
        ].
        idx := idx - 1.
        firstRealIndex := 2.
    ].

    [line := listView at:firstRealIndex. 
     (line startsWith:'-') and:[line size < 2 or:[line second isDigit not]]] whileTrue:[
        firstRealIndex := firstRealIndex + 1.
        idx := idx - 1.
    ].

    cls := inspectedObject class.
    baseCls := self baseInspectedObjectClass.

    nNamedInstvarsShown := cls instSize.
    "/ only the namedInstvars below baseInspectedObjectClass
    "/ are shown ...
    (cls == baseCls or:[cls isSubclassOf:baseCls]) ifTrue:[
        nNamedInstvarsShown := nNamedInstvarsShown - baseCls instSize.
    ].

    idx := idx - nNamedInstvarsShown.
    idx < 1 ifTrue:[
        ^ nil.
    ].

    nExtraNamedInstvarsShown := self extraNamedFields size.
    idx <= nExtraNamedInstvarsShown ifTrue:[
        ^ idx.
    ].

    ^ nil "/ indexed instvar or other selected

    "Created: / 03-08-2006 / 13:45:14 / cg"
    "Modified: / 04-08-2006 / 11:45:29 / cg"
!

fieldList 
    "return a list of names to show in the selectionList.
     Leave hasMore as true, if a '...' entry should be added."

    |derivedFieldList namedFieldList fieldList cls indexedList extraNamedFieldList|

    inspectedObject isNil ifTrue:[
        ^ self hasSelfEntry ifFalse:[ #() ] ifTrue:[ #('-self') ]
    ].

    cls := inspectedObject class.

    self topView withWaitCursorDo:[
        namedFieldList := self namedFieldList.
        indexedList := self indexedFieldList.
        extraNamedFieldList := OrderedCollection new.

        self hasSelfEntry ifTrue:[
            self suppressPseudoSlots ifFalse:[
                derivedFieldList := OrderedCollection new.
                derivedFieldList add:'-self'.
                derivedFieldList add:'-hash'.
                derivedFieldList add:'-identityHash'.

                cls hasImmediateInstances ifFalse:[
                    derivedFieldList add:'-dependents'.
                ].
                cls instSize > 0 ifTrue:[
                    derivedFieldList add:'-all inst vars'.
                ].
                (indexedList notNil and:[self showAllIndexedVarsInFieldList]) ifTrue:[
                    derivedFieldList add:'-all indexed vars'.
                ].

                derivedFieldList addAll:(self derivedFieldNames).
                extraNamedFieldList addAll:(self extraNamedFieldNames).
            ].
        ].

        fieldList := OrderedCollection new.
        derivedFieldList notNil ifTrue:[fieldList addAll:derivedFieldList].
        namedFieldList notNil ifTrue:[fieldList addAll:namedFieldList].
        extraNamedFieldList notNil ifTrue:[fieldList addAll:extraNamedFieldList].
        indexedList notNil ifTrue:[fieldList addAll:indexedList].
    ].
    ^ fieldList

    "Modified: / 18-09-2006 / 21:16:03 / cg"
!

hasSelfEntry
    ^ hideReceiver not and:[self suppressPseudoSlots not]

    "Created: 14.12.1995 / 19:30:03 / cg"
    "Modified: 28.6.1996 / 15:13:41 / cg"
!

indexList 
    "return a list of indexes to show in the selectionList.
     Set hasMore to true, if a '...' entry should be added."

    | n cls|

    cls := inspectedObject class.

    cls isVariable ifFalse:[^ nil ].

    n := inspectedObject basicSize.
    (n > nShown) ifTrue:[
        n := nShown.
        hasMore := true.
    ].

    ^ (1 to:n)
!

indexedFieldList 
    "return a list of indexed-variable names to show in the selectionList.
     Set hasMore to true, if a '...' entry should be added."

    |l|

    l := self indexList.
    l isNil ifTrue:[^ nil ].
    ^ l collect:[:i | i printString].
!

indexedValueAtIndex:idx
    ^ inspectedObject basicAt:idx
!

indexedValueAtIndex:idx put:newValue
    inspectedObject basicAt:idx put:newValue
!

indexedValueAtKey:key
    "/ kludge
    inspectedObject isLimitedPrecisionReal ifTrue:[
        ^ inspectedObject basicAt:key
    ].
    ^ inspectedObject at:key
!

instVarIndexForLine:lineNr
    "helper - return the index for a named instVar;
     nil, if self or a keyed instvar is selected."

    |idx nNamedInstvarsShown cls baseCls firstRealIndex line|

    lineNr isNil ifTrue:[^ nil].
    firstRealIndex := 1.

    idx := lineNr.
    self hasSelfEntry ifTrue:[
        (lineNr == 1 or:[lineNr isNil]) ifTrue:[
            ^ nil "/ self selected
        ].
        idx := idx - 1.
        firstRealIndex := 2.
    ].

    [line := listView at:firstRealIndex. 
     (line startsWith:'-') and:[line size < 2 or:[line second isDigit not]]] whileTrue:[
        firstRealIndex := firstRealIndex + 1.
        idx := idx - 1.
    ].

    cls := inspectedObject class.
    baseCls := self baseInspectedObjectClass.

    nNamedInstvarsShown := cls instSize.
    "/ only the namedInstvars below baseInspectedObjectClass
    "/ are shown ...
    (cls == baseCls or:[cls isSubclassOf:baseCls]) ifTrue:[
        nNamedInstvarsShown := nNamedInstvarsShown - baseCls instSize.
    ].

    idx <= nNamedInstvarsShown ifTrue:[
        ^ idx + self baseInspectedObjectClass instSize.
    ].
    ^ nil "/ indexed instvar or other selected

    "Modified: / 31.10.2001 / 09:20:20 / cg"
!

keyIndexForLine:lineNr
    "helper - return the index of the key-list;
     nil, if self or a namedInstVar is selected."

    |idx nNamedInstvarsShown nExtraNamedInstvarsShown cls baseCls firstRealIndex line|

    lineNr isNil ifTrue:[^ nil].

    firstRealIndex := 1.
    idx := lineNr.
    self hasSelfEntry ifTrue:[
        (lineNr == 1 or:[lineNr isNil]) ifTrue:[
            ^ nil "/ self selected
        ].
        idx := idx - 1.
        firstRealIndex := firstRealIndex + 1.
    ].

    [line := listView at:firstRealIndex. 
     line notNil
     and:[(line startsWith:'-') and:[line size < 2 or:[line second isDigit not]]]
    ] whileTrue:[
        firstRealIndex := firstRealIndex + 1.
        idx := idx - 1.
    ].

    cls := inspectedObject class.
    baseCls := self baseInspectedObjectClass.

    nNamedInstvarsShown := cls instSize.
    "/ only the namedInstvars below baseInspectedObjectClass
    "/ are shown ...
    (cls == baseCls or:[cls isSubclassOf:baseCls]) ifTrue:[
        nNamedInstvarsShown := nNamedInstvarsShown - baseCls instSize.
    ].
    nExtraNamedInstvarsShown := self extraNamedFields size.

    idx <= (nNamedInstvarsShown+nExtraNamedInstvarsShown) ifTrue:[
        ^ nil "/ named instVar selected.
    ].
    ^ idx - (nNamedInstvarsShown+nExtraNamedInstvarsShown).

    "Modified: / 04-08-2006 / 11:45:42 / cg"
!

namedFieldList 
    "return a list of instVar names to show in the selectionList."

    |aList cls baseCls|

    cls := inspectedObject class.
    baseCls := self baseInspectedObjectClass.

    aList := OrderedCollection new.

    aList addAll:(cls allInstVarNames).
    (cls == baseCls or:[cls isSubclassOf:baseCls]) ifTrue:[
        "/ hide some stuff
        aList := aList copyFrom:(self baseInspectedObjectClass instSize + 1).
    ].

    ^ aList

    "Modified: / 18-09-2006 / 21:35:30 / cg"
!

setAcceptAction
    "set the codeViews accept action"

    |acceptAction sel|

    acceptAction := [:theText | self doAccept:theText asString].

    (selectionIndex isNil 
    or:[ inspectedObject class hasImmediateInstances])
    ifTrue:[
        acceptAction := nil.
    ] ifFalse:[
        sel := listView at:selectionIndex.

        (sel startsWith:'-all') ifTrue:[
            acceptAction := nil.
        ].
        (sel startsWith:'-hash') ifTrue:[
            acceptAction := nil.
        ].
        (sel startsWith:'-identityHash') ifTrue:[
            acceptAction := nil.
        ].
        (sel startsWith:'-dependents') ifTrue:[
            acceptAction := nil.
        ].
        (sel startsWith:'-source') ifTrue:[
            acceptAction := nil.
        ].
    ].

    workspace acceptAction:acceptAction.
!

setDoItAction
    "set the codeViews doit action"

    workspace 
	doItAction:[:theCode |
	    |evaluator|

	    (evaluator := inspectedObject class evaluatorClass)
	    notNil ifTrue:[
		evaluator
		    evaluate:theCode 
		    in:nil 
		    receiver:inspectedObject 
		    notifying:workspace 
		    logged:true 
		    ifFail:nil
	    ] ifFalse:[
		'objects class provides no evaluator'
	    ]
	].

    inspectedObject class evaluatorClass isNil ifTrue:[
	workspace doItAction:nil.
	workspace acceptAction:nil.
    ]

    "Modified: 1.8.1997 / 21:47:09 / cg"
!

showAllIndexedVarsInFieldList
    ^ true
!

showMore
    |o|

    hasMore ifTrue:[
        nShown := nShown * 2.
        "/ force update (which is otherwise ignored)
        o := inspectedObject.
        inspectedObject := object := nil.
        self inspect:o
    ]

    "Modified: / 26.8.1998 / 19:05:25 / cg"
!

showSelection:lineNr
    "user clicked on an instvar - show value in workspace"

    |val|

    (hasMore and:[lineNr == listView list size]) ifTrue:[
        "clicked on the '...' entry"
        self showMore.
        listView setSelection:lineNr.
    ].

    self hasSelfEntry ifTrue:[
        (self hasSelfEntry and:[lineNr == 1]) ifTrue:[
            "selecting self also does a re-set, this allows updating the list"
            self reinspect.
        ].
    ].
    selectionIndex := selectedLine := lineNr.
    val := self selection.
    self showValue:val.

    self setAcceptAction.

    "Modified: / 03-08-2006 / 14:26:22 / cg"
!

showValue:someValue 
    "user clicked on an entry - show value in workspace"

    |s|

    self topView withWaitCursorDo:[
        s := self displayStringForValue:someValue.
        s = workspace selectionAsString ifFalse:[
            workspace replace:s.
        ].
    ].

    "Modified: / 06-02-2007 / 14:26:47 / cg"
!

stringWithAllIndexedVarValues
    |nIdx s names maxLen varString padLeft|

    nIdx := inspectedObject size.

    s := '' writeStream.
    names := self indexList.
    names size > 0 ifTrue:[
        maxLen := (names collect:[:eachName | eachName printString size]) max.
        padLeft := names conform:[:eachIdx | eachIdx isInteger].
        
        names do:[:eachIdx |
            |val|

            padLeft ifTrue:[
                s nextPutAll:(eachIdx printStringLeftPaddedTo:maxLen).
            ] ifFalse:[
                s nextPutAll:((eachIdx printString , ' ') paddedTo:maxLen+1 with:$.).
            ].

            s nextPutAll:' : '.
            val := self indexedValueAtKey:eachIdx.

            varString := self basicDisplayStringForValue:val.
            (varString includes:Character cr) ifTrue:[
                varString := varString copyTo:(varString indexOf:Character cr)-1.
                varString := varString , '...'.
            ].
            s nextPutLine:varString.
        ].
    ].
    nShown < nIdx ifTrue:[
        s nextPutLine:' ...'.
    ].
    ^ s contents
!

stringWithAllInstVarValues
    |s names maxLen varString|

    s := WriteStream on:Unicode32String new.
    names := inspectedObject class allInstVarNames.
    maxLen := (names collect:[:eachName | eachName size]) max.
    names keysAndValuesDo:[:eachInstVarIndex :eachInstVarName |
        s nextPutAll:((eachInstVarName , ' ') paddedTo:maxLen+1 with:$.).
        s nextPutAll:' : '.
        varString := self basicDisplayStringForValue:(inspectedObject instVarAt:eachInstVarIndex).
        (varString includes:Character cr) ifTrue:[
            varString := varString copyTo:(varString indexOf:Character cr)-1.
            varString := varString , '...'.
        ].
        s nextPutAll:varString.
        s cr.
    ].
    ^ s contents asSingleByteStringIfPossible
!

valueAtLine:lineNr
    "helper - return the value of the selected entry"

    |idx l val|

    (self hasSelfEntry
    and:[lineNr == 1 or:[lineNr isNil]]) ifTrue:[
        ^ inspectedObject
    ].

    ((l := listView at:lineNr) startsWith:$-) ifTrue:[
        (l ~= '-' and:[(l at:2) isDigit not "negative number"]) ifTrue:[
            ^ self valueForSpecialLine:(listView at:lineNr)
        ].
    ].

    "/ a named instVar ?
    idx := self instVarIndexForLine:lineNr.
    idx notNil ifTrue:[
        BreakPointInterrupt catch:[
            ^ inspectedObject instVarAt:idx
        ]
    ].

    "/ an extra named field ?
    idx := self extraNamedVarIndexForLine:lineNr.
    idx notNil ifTrue:[
        BreakPointInterrupt catch:[
            val := ((self extraNamedFields) at:idx) value.
            val isBlock ifTrue:[ val := val value ].
            ^ val
        ]
    ].

    "/ an indexed instVar ?
    idx := self keyIndexForLine:lineNr.
    idx notNil ifTrue:[
        BreakPointInterrupt catch:[
            ^ self indexedValueAtIndex:idx.
        ]
    ].

    "/ nope
    ^ nil

    "Modified: / 03-08-2006 / 14:05:30 / cg"
!

valueAtLine:lineNr put:newValue
    |idx|

    idx := self instVarIndexForLine:selectionIndex.
    idx notNil ifTrue:[
        inspectedObject instVarAt:idx put:newValue.
    ] ifFalse:[
        idx := self keyIndexForLine:selectionIndex.
        idx notNil ifTrue:[
            self indexedValueAtIndex:idx put:newValue.
        ] ifFalse:[
            ^ self "/ self selected - dont store
        ]
    ].
!

valueForSpecialLine:line
    |idx fieldEntry val extraAttributes|

    extraAttributes := inspectedObject inspectorExtraAttributes.
    (extraAttributes includesKey:line) ifTrue:[
        ^ (extraAttributes at:line) value 
    ].

    idx := self derivedFieldNames indexOf:line.
    idx ~~ 0 ifTrue:[
        fieldEntry := self derivedFields at:idx.
    ] ifFalse:[
        idx := self extraNamedFieldNames indexOf:line.
        idx ~~ 0 ifTrue:[
            fieldEntry := self extraNamedFields at:idx.
        ].
    ].
    fieldEntry notNil ifTrue:[
        fieldEntry isAssociation ifTrue:[
            ^ fieldEntry value
        ].
        ^ fieldEntry at:2.
    ].

    (line startsWith:'-self') ifTrue:[
        ^ inspectedObject
    ].
    (line startsWith:'-hash') ifTrue:[
        ^ inspectedObject hash
    ].
    (line startsWith:'-identityHash') ifTrue:[
        ^ inspectedObject identityHash
    ].
    (line startsWith:'-dependents') ifTrue:[
        ^ inspectedObject dependents
    ].
    (line startsWith:'-all') ifTrue:[
        ^ inspectedObject
    ].

    self error:'unknown special line'.

    "Created: / 31-10-2001 / 09:17:45 / cg"
    "Modified: / 18-09-2006 / 21:34:54 / cg"
! !

!InspectorView methodsFor:'queries'!

canInspect:anObject
    ^ anObject inspectorClass == self class
!

compilerClass
    ^ inspectedObject class compilerClass
!

dereferenceValueHolders
    ^ dereferenceValueHolders ? false
!

labelFor:anObject
    "return the windowLabel to use in my topView, when inspecting anObject."

    |lbl id|

    (self class == InspectorView
    and:[anObject inspectorClass ~~ InspectorView]) ifTrue:[
        lbl := 'BasicInspector on: '
    ] ifFalse:[
        lbl := 'Inspector on: '
    ].
    (anObject isImmediate
    or:[anObject isBoolean]) ifFalse:[
        lbl := lbl , '<%2> ' 
    ].
    lbl := lbl , '%1'.

    IdDictionary isNil ifTrue:[
        IdDictionary := WeakIdentityDictionary new.
    ].
    [    
        id := IdDictionary 
                    at:anObject 
                    ifAbsentPut:[ 
                        |nextID|

                        nextID := NextSequentialID ? 0.
                        NextSequentialID := nextID + 1.
                        nextID
                    ].
    ] valueUninterruptably.

    ^ self class classResources 
        string:lbl 
        with:(self labelNameFor:anObject)
        with:id
!

labelNameFor:anObject
    "return the iconLabel to use in my topView, when inspecting anObject.
     Simply returns the className or name of anObjects class"

    |s|

    anObject isClass ifTrue:[
        s := anObject displayString
    ] ifFalse:[
        ((anObject class == SmallInteger)
        or:[anObject isBoolean]) ifTrue:[
            s := anObject printString , ', ' , anObject classNameWithArticle
        ] ifFalse:[
            s := anObject classNameWithArticle
        ].
    ].
    s isNil ifTrue:[
        anObject isBehavior ifTrue:[
            ^ 'someBehavior'
        ].
        ^ 'something'
    ].
    ^ s

    "
     1234 inspect
     true inspect
     $a inspect
    "
!

selectedKeyName
    selectionIndex notNil ifTrue:[
        ^ listView listAt:selectionIndex.
    ].
    ^ nil
!

suppressPseudoSlots
    ^ suppressPseudoSlots ? false
! !

!InspectorView methodsFor:'user interaction'!

doAccept:theText
    |sel newValue|

    sel := listView at:selectionIndex.
    (sel startsWith:'-all') ifTrue:[
        workspace flash.
        ^ self.
    ].

    Error handle:[:ex |
        workspace flash
    ] do:[
        newValue := inspectedObject class evaluatorClass 
                       evaluate:theText
                       receiver:inspectedObject 
                       notifying:workspace.

        self dereferenceValueHolders ifTrue:[
            (self valueAtLine:selectionIndex) value:newValue
        ] ifFalse:[
            self valueAtLine:selectionIndex put:newValue.
        ]
    ]
!

doCopyKey
    "put the instVar-name into the text-copy-buffer"

    |nm|

    selectionIndex notNil ifTrue:[
        nm := listView listAt:selectionIndex.
        nm notNil ifTrue:[
            self setClipboardText:(nm asString)
        ]
    ]
!

doInspect:basic
    "user selected inspect-menu entry"

    |objectToInspect|

    selectionIndex notNil ifTrue:[
        objectToInspect := self selection.
        basic == #new ifTrue:[
            NewInspector::NewInspectorView inspect:objectToInspect
        ] ifFalse:[
            basic ifTrue:[
                objectToInspect basicInspect
            ] ifFalse:[
                objectToInspect inspect
            ]
        ]
    ]

    "Modified: / 31.10.1997 / 12:46:53 / cg"
!

doUpdate
    self reinspect
!

keyPress:aKey x:x y:y
    "all my input is passed on to the workspace-field"

    x notNil ifTrue:[
	"/ not already delegated

	workspace keyPress:aKey x:-1 y:-1
    ].

    "Modified: 4.3.1996 / 22:21:37 / cg"
!

monitor:anInstVarName
    "start a monitoring process, showing the given instVar
     in regular intervals."

    |ivName|

    (ivName := anInstVarName) isInteger ifTrue:[
        ivName := anInstVarName printString
    ].
    listView selectElement:ivName.
    self doStartMonitor

    "Created: / 1.3.1996 / 19:31:45 / cg"
    "Modified: / 12.2.1999 / 16:05:47 / cg"
!

selection
    "helper - return the value of the selected entry"

    |val|

    selectionIndex isNil ifTrue:[^ nil].

    val := self valueAtLine:selectionIndex.
    self dereferenceValueHolders ifTrue:[
        "workspace-variable-inspection"
        val := val value
    ].
    ^ val

    "Modified: / 03-08-2006 / 14:27:02 / cg"
!

showLast
    "user clicked on an instvar - show value in workspace"

    |lastIdx|

    lastIdx := listView list size.
    lastIdx ~~ 0 ifTrue:[
	self showSelection:lastIdx.
	listView selection:lastIdx.
    ]

    "Created: 28.6.1996 / 15:06:38 / cg"
    "Modified: 18.3.1997 / 18:22:54 / cg"
!

showReferences
    "user selected references-menu entry"

    self selection class hasImmediateInstances ifTrue:[
        ^ self warn:'Sorry - cannot show references to immediate objects'
    ].
    ObjectMemory displayRefChainTo:(self selection)

    "Modified: / 30.7.1998 / 14:03:16 / cg"
!

toggleHex
    |sel|

    showHex := showHex not.
    sel := listView selection.
    sel notNil ifTrue:[
	self showSelection:sel
    ]

    "Created: / 7.5.1998 / 01:54:52 / cg"
    "Modified: / 7.5.1998 / 02:00:10 / cg"
!

tryToSelectKeyNamed:aString
    |list idx|

    list := listView list.
    list notNil ifTrue:[
        (idx := list indexOf:aString) ~= 0 ifTrue:[
            listView selection:idx
        ].
    ].

    "Created: / 16.11.2001 / 13:48:51 / cg"
! !

!InspectorView methodsFor:'workspace protocol'!

modified:aBoolean
    ^ workspace modified:aBoolean
!

saveAs:file doAppend:doAppend
    workspace saveAs:file doAppend:doAppend
! !

!InspectorView class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/libtool/InspectorView.st,v 1.175 2007-12-06 21:40:25 stefan Exp $'
! !
